/**
 * apeman app to send response.
 * @memberof module:apeman-app-send/lib
 * @function create
 * @param {object} [options] - Optional settings.
 * @param {string} [options.name='send'] - Name of property for request.
 * @returns {function} - Defined app function.
 */

"use strict";

const argx = require('argx'),
    objnest = require('objnest');

/** @lends create */
function create(options) {
    let args = argx(arguments);
    options = objnest.expand(args.pop('object') || {});

    let name = options.name || 'send';


    /**
     * Defined app.
     * @function app
     * @param {object} req - Request object.
     * @param {object} res - Response object.
     * @param {function} next - Pass to next handler.
     */
    function app(req, res, next) {

        if (!req.format) {
            throw new Error('req.format not found. Use "apeman-app-format" before this.');
        }

        function send(data, format) {
            format = format || req.format;
            let isValid = res[format] && typeof (res[format]) === 'function';
            if (isValid) {
                res[format](data);
            } else {
                throw new Error('req.' + format + 'not found. Try "apeman-app-' + format + '" or some other middlewares.');
            }
        }

        function _status(statusCode, data, format) {
            res.statusCode = statusCode;
            send(data, format);
        }

        // Handy calls
        Object.assign(send, {
            _status: _status,
            ok: _status.bind(this, 200),
            bad: _status.bind(this, 400),
            notfound: _status.bind(this, 404),
            unauthorized: _status.bind(this, 401)
        });

        res[name] = res[name] || send;

        next();
    }

    // Description of this app.
    app.$desc = "Add function to send response.";
    return app;
}

module.exports = create;
